package com.getsnappy.snappyj;

import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.getsnappy.util.ReflectionCache.ConstructorCache;


/**
 * This class as a wrapper around HttpServlet is slightly 
 * more convenient and plays nicer with Eclipse than the HttpServlet 
 * class.  This class is strictly optional, but I find the
 * error handling mechanisms quite useful.
 * 
 * @author brian
 *
 */
public abstract class ApplicationInstance extends HttpServlet
{
   private ApplicationErrorHandler errorHandler = new DefaultApplicationErrorHandler();
   
   /**
    * Specify an alternate error handler.  The handleError method of this
    * error handler is called any time an exception is thrown out of output
    * 
    * @param error is the new ApplicationErrorHandler
    */
   public void setApplicationErrorHandler(ApplicationErrorHandler error) {
      this.errorHandler = errorHandler;
   }
   
   /**
    * This abstract class must be overriden and is called whenever this 
    * servlet is excecuted.
    * 
    * @param out
    * @param request
    * @param response
    * @throws Exception
    */
   public abstract void output(Writer out, HttpServletRequest request, javax.servlet.http.HttpServletResponse response) throws Exception;
   
   /**
    * simply forward to the output function and turn exceptions that
    * are not already of type IOException or ServletException into
    * ServletExceptions before passing them back out.
    */
   public void doGet(HttpServletRequest request, 
         HttpServletResponse response)
   throws IOException, ServletException{
      try {
         try {
            
            output(response.getWriter(),request,response);
         } catch (Exception e) {
            errorHandler.handleException(e, response.getWriter(), request, response);
         }
      } catch (IOException e) {
         throw e;
      } catch (ServletException e) {
         throw e;
      } catch (Exception e) {
         throw new ServletException(e);
      }
   }

   /**
    * simply forward to the output function and turn exceptions that
    * are not already of type IOException or ServletException into
    * ServletExceptions before passing them back out.
    */
   public final void doPost(HttpServletRequest request, 
         HttpServletResponse response)
   throws IOException, ServletException {
      doGet(request,response);
   }
}