package com.getsnappy.snappyj;

import java.io.IOException;
import java.io.Writer;

import com.sun.org.apache.xpath.internal.functions.Function;

/**
 * This loading indicator  will apply the opaque png "/images/grey.png" screenwide and
 * then load /images/loadingBox.png and attempt to keep it centered in the screen
 *
 * @author brian
 *
 */
public class ContextAnimatedGifLoadingIndicator extends JavaScriptLoadingIndicator {
   private static ContextAnimatedGifLoadingIndicator indicator;
   private String contextId;
   private boolean useAbsolutePositioning = false;
   private String imageUri;
   private int imageWidth;
   private int imageHeight;
   private int distanceFromTop = -1;
   private int distanceFromLeft = -1;
   
   

   /**
    * Creates an loading image with the default imageUri /images/loadingBox.png
    * @param contextId
    */
   public ContextAnimatedGifLoadingIndicator(String contextId) { 
      this(contextId, "/images/ajax-loader-big.gif", 32, 32);
   }

   /**
    * Will display the following image with the specified width and height in the center 
    * of the context if possible.  If the height of the image is larger then the context they
    * are aligned at top edges.  If the width of the image is larger then the context width, the
    * image should be aligned with the left edge of the context.
    * 
    * @param contextId - the id of an html element
    * @param imageUri - the uri for an image
    * @param imageWidth - the width of the image
    * @param imageHeight - the height of the image
    */
   public ContextAnimatedGifLoadingIndicator(String contextId, String imageUri, int imageWidth,int imageHeight) { 
      this.contextId = contextId;
      this.imageUri = imageUri;
      this.imageWidth = imageWidth;
      this.imageHeight = imageHeight;

   }

   /**
    * Will display the following image with the specified width and height.
    * Placement of the Loading Indicator gif is controlled by the 
    * distanceFromTop and distanceFromLeft parameters
    * 
    * @param contextId - the id of an html element
    * @param imageUri - the uri for an image
    * @param imageWidth - the width of the image
    * @param imageHeight - the height of the image
    * @param top - the value used for absolute positioning css attribute top.   
    * 
    * null will result in this attribute not being set
    */
   public ContextAnimatedGifLoadingIndicator(String contextId, String imageUri, int imageWidth,int imageHeight,
         int distanceFromTop, int distanceFromLeft) { 
      this.contextId = contextId;
      this.imageUri = imageUri;
      this.imageWidth = imageWidth;
      this.imageHeight = imageHeight;
      this.distanceFromTop = distanceFromTop;
      this.distanceFromLeft = distanceFromLeft;
   }
 
   
   /**
    * This function will apply the opaque png "/images/grey.png" screenwide and
    * then load /images/loadingBox.png and attempt to keep centered in the screen
    * 
    * @throws IOException 
    */
   @Override
   public void jsStart(Writer out) throws IOException {
      
      out.write("function () {\n");
      out.write("   var context = document.getElementById(" + StringUtil.jsQuote(contextId) + ");\n");
      out.write("   if (context.firstChild && context.firstChild.id != " + StringUtil.jsQuote(contextId + "LI") + ") {\n");
      out.write("      var loadingBox2 = document.createElement('div');\n");
      out.write("      loadingBox2.style.position = 'absolute';\n");
      
      if (distanceFromTop == -1 && distanceFromLeft == -1) {
         out.write("      var cWidth = (context.clientWidth != 0 ? context.clientWidth : context.scrollWidth)\n");
         out.write("      var cHeight = (context.clientHeight != 0 ? context.clientHeight : context.scrollHeight)\n");
         out.write("      var contextLeft = '' + ((cWidth ) / 2 - " + ((int)imageWidth / 2) + ") + 'px';\n");
         out.write("      var contextTop = '' + (cHeight / 2 - " + ((int)imageHeight / 2) + ") + 'px';\n");
         out.write("      loadingBox2.style.marginLeft = contextLeft;\n");
         out.write("      loadingBox2.style.marginTop = contextTop;\n");
      } else {
         out.write("      loadingBox2.style.marginLeft = '" + distanceFromLeft + "px';\n");
         out.write("      loadingBox2.style.marginTop = '" + distanceFromTop + "px';\n");
      }
      out.write("      loadingBox2.style.background = 'url(" + imageUri + ") no-repeat';\n");
      out.write("      loadingBox2.style.width='"+ imageWidth + "px';\n");
      out.write("      loadingBox2.style.height='" + imageHeight + "px';\n");
 
      out.write("      loadingBox2.id=" + StringUtil.jsQuote(contextId + "LI") + ";\n");
      out.write("      if (!context.firstChild) {\n");
      out.write("         context.appendChild(loadingBox2);\n");
      out.write("      } else {\n");
      out.write("         context.insertBefore(loadingBox2,context.firstChild);\n");
      out.write("      }\n");
      out.write("   }\n");
      
      out.write("}");
   }
   
   @Override
   public void jsFinish(Writer out) throws IOException {
      out.write("function () {\n");
      out.write("   var contextLI = document.getElementById(" + StringUtil.jsQuote(contextId + "LI") + ");\n");
      out.write("   if (contextLI != undefined) {\n");
      out.write("      contextLI.parentNode.removeChild(contextLI);\n");
      out.write("   }\n");
      
      out.write("}");
   }
}
