package com.getsnappy.snappyj;

import java.io.IOException;
import java.io.Writer;

/**
 * This is used to indicate to a user using JavaScript that
 * it's containing context is being reloaded or forwarded.
 * Under the hood we make an 
 * asynchronous call to replace the contents of the context
 * but it seems like bad user interface to just let the 
 * client wait without displaying some sort of indication to
 * them that we are performing an asynchronous action and that
 * your servers are busy generating results.  By default
 * asynchronous requests do not display a busy pointer, so
 * it appears to the client that they did not
 * click anything and nothing is happening. This class can be
 * easily extended to support your exact needs.  We provide several 
 * useful implementations out of the box.<br>
 * <br>
 * [Small|Medium|Large]AnimatedGifLoadingIndicator - Display an animated gif (or any image) 
 * centered in the space of this context.
 *  DoNothingLoadingIndicator -
 * 
 */
public abstract class JavaScriptLoadingIndicator {
   /**
    * This function is called immediately before reloading or forwarding
    * a context from JavaScript.  This is abstract because it should always
    * do something, except for the DoNothingLoadingIndicator, which true to
    * it's name does nothing.  It must output the JavaScript definition of a
    * function (everything BETWEEN the braces).
    * 
    * 
    * @param out
    * @throws IOException 
    * @see DoNothingLoadingIndicator
    */
   public abstract void jsStart(Writer out) throws IOException;
   

   /**
    * Called immediately before replacing the contents of the page.  One use is for
    * for displaying 100% on a progress.  Another concept is too allow browsers to 
    * load all images and/or other external resources before showing the contents.
    * It must output the JavaScript definition of a function (everything BETWEEN the braces).
    *
    * @param out
    * @throws IOException 
    */
   public void jsFinish(Writer out) throws IOException {
      out.write("undefined");
   }
// FIXME: implement jsFinish and maybe jsProgress for progress bars
}
