package com.getsnappy.snappyj;

/**
 * This class provides central documentation to all options applicable to 
 * the web 2.0 framework library.  Currenly you can define:<br>
 * -Dcom.getsnappy.ServletForwarderUrl="/servletForwarder" - specify the location of the
 * ServletForwarder servlet<br>
 * -Dcom.getsnappy.ClassNamePrefix="" - specify a prefix thats prepended to any className
 * passed through the API
 * 
 * @see ServletForwarder
 * @author Brian Gardner 
 */
public class Options {
   private static final String CLASS_NAME_PREFIX="com.getsnappy.ClassNamePrefix";
   private static final String SERVLET_FORWARDER_URL="com.getsnappy.ServletForwarderUrl";
   private static String classNamePrefix = "";
   private static String errorHandlerClass = "com.getsnappy.w20.DefaultJavaScriptErrorHandler";
   private static String servletForwarderUrl = "/servletForwarder";
   static {
      String l_classNamePrefix = System.getProperty(CLASS_NAME_PREFIX);
      if (l_classNamePrefix != null) {
         classNamePrefix = l_classNamePrefix;
         if (classNamePrefix.charAt(classNamePrefix.length() - 1) != '.') 
            classNamePrefix += '.';
            
      }
         
      String l_servletForwarderUrl = System.getProperty(SERVLET_FORWARDER_URL);
      if (l_servletForwarderUrl != null)
         servletForwarderUrl = l_servletForwarderUrl;
         
   }
   /**
    * If -Dcom.getsnappy.ClassNamePrefix is defined, it will prefix this value
    * to className (adding a "." in between them if necessary).  For example 
    * we set Dcom.getsnappy.ClassNamePrefix="com.getsnappy".
    * Then when we specify a className to either a StaticContext or a CategoryChooserBean
    * we would simply pass "report.ReportWizard" and this className
    * would resolve to "com.getsnappy.report.ReportWizard"
    * 
    * @param className
    * @return the param className prepended by -Dcom.getsnappy.ClassNamePrefix
    */
   public static String getCompleteClassName(String className) {
      return classNamePrefix + className;
   
   }
   
   /**
    * If -Dcom.getsnappy.ServletForwarder is defined, it returns this value,
    * otherwise it default /servletForwarder.  You must define entries for
    * com.getsnappy.w20.ServletForwarder, only if you intend to use web 2.0
    * loading method functionality.
    * 
    * @see Web20Environment
    */
   public static String getServletForwarderUrl() {
      return servletForwarderUrl;
      
   }

   /**
    * Used to wrap JavaScript statements that are useful for debugging issues with
    * the library.  Returns "" if errorHandlerClass is null, otherwise returns the
    * String js without modifying it.  
    * 
    * Example:
    * out.write(Options.DEBUG("if (e == undefined) { w20.handleError('The var e is undefined'); }"))"
    * 
    * @param js
    * @return
    */
   public static String DEBUG(String js) {
      if (errorHandlerClass != null) {
         return  js;
      }
      return "";
   }
   
   /**
    * This class contains the functionality to alert a user when something
    * unexpected happens.
    * @return
    */
   public static String getErrorHandlerClass() {
      return errorHandlerClass;
      
   }
}
