package com.getsnappy.snappyj;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URLEncoder;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import com.getsnappy.core.InvalidParameterException;
import com.getsnappy.util.ReflectionCache;
import com.getsnappy.util.ReflectionCache.ConstructorCache;
import com.getsnappy.util.ReflectionCache.MethodCache;
import com.sun.mail.iap.Response;

/**
 * This class is used to forward all web 2.0 requests to the
 * appropriate destination.  If it is not configured in your
 * web.xml file, your application server will most likely get
 * a 404 on requests made through the web 2.0 loading methods.
 * Below I've included sample entries that need to be nested
 * directly underneath the web-app tag.
 * 
 * <servlet>  
 *   <servlet-name>ServletForwarder</servlet-name>  
 *   <servlet-class>com.getsnappy.w20.ContextServlet</servlet-class>
 * </servlet> 
 * <servlet-mapping>  
 *   <servlet-name>ServletForwarder</servlet-name>  
 *   <url-pattern>/servletForwarder</url-pattern>
 * </servlet-mapping> 
 * 
 * @author brian
 * @see Web20Environment
 */
public class ServletForwarder extends HttpServlet {
   public static class Parameters {
      public static final String IFRAME_POS = "W20IFramePos";
      public static final String TITLE = "W20Title";
   }
   
   Logger log = Logger.getLogger(ServletForwarder.class);
   private static final long serialVersionUID = 1L;
 
   private Class httpServlet = null;
   private String className = null;
   
   
   
   /**
    * This type of ServletForwarder will be use the request passed to
    * doGet or doPost, and load the className from the parameter 
    * applicationClassName
    */
   public ServletForwarder() {      
   }
   
   /**
    * Define the className to forward.  This should resolve to an 
    * ApplicationInstance or HttpServlet instance.
    * 
    * NOTE: the getCompleteClassName will be applied to this className
    * @param className
    */
   public ServletForwarder(String className) {
      this.className = className;
   }
   
   /**
    * Define the ApplicationInstance or HttpServlet to forward. 
    * 
    * NOTE: the getCompleteClassName will be applied to this className
    * @param className
    */
   public ServletForwarder(Class httpServlet) {
      this.httpServlet = httpServlet;
   }
   
   /**
    * Call doGet in the underlying servlet defined by the constructor
    * or the one specified by the applicationClassName request parameter
    * if the default contructor was used.
    */
   public void doGet(HttpServletRequest request, 
         HttpServletResponse response) throws IOException, ServletException{
      Class appClass = getApplicationClass(request);
      if (appClass == null) {
         if (outputHiddenIFrame(request, response)) {
            return;
         } else {
            throw new ServletException("Must specify an applicatonClass or state and title");
         }
      }
      
      if (HttpServlet.class.isAssignableFrom(appClass)) {
         try {
            Constructor<HttpServlet> contextContructor = ConstructorCache.getConstructor(appClass);
            HttpServlet  servlet = contextContructor.newInstance();
            Method doGet = MethodCache.getMethod(servlet.getClass(), "doGet", HttpServletRequest.class, HttpServletResponse.class);
            doGet.setAccessible(true);
            doGet.invoke(servlet,request,response);
         } catch (SecurityException e) {
            throw new ServletException(e);
         } catch (NoSuchMethodException e) {
            throw new ServletException(e);
         } catch (IllegalArgumentException e) {
            throw new ServletException(e);
         } catch (InstantiationException e) {
            throw new ServletException(e);
         } catch (IllegalAccessException e) {
            throw new ServletException(e);
         } catch (InvocationTargetException e) {
            log.error(null,e);
            throw new ServletException(e);
         }
         
      }
   }

   /**
    * Call doPost in the underlying servlet defined by the constructor
    * or the one specified by the applicationClassName request parameter
    * if the default contructor was used.
    */
   public void doPost(HttpServletRequest request, 
         HttpServletResponse response)
   throws IOException, ServletException {
      
      Class appClass = getApplicationClass(request);
      if (appClass == null) {
         if (outputHiddenIFrame(request, response)) {
            return;
         } else {
            throw new ServletException("Must specify an applicatonClass or state and title");
         }
      }
            
      if (HttpServlet.class.isAssignableFrom(appClass)) {
         try {
            Constructor<HttpServlet> contextContructor = ConstructorCache.getConstructor(appClass, HttpServletRequest.class, HttpServletResponse.class);
            HttpServlet  servlet = contextContructor.newInstance();
            Method doPost = MethodCache.getMethod(appClass, "doPost", HttpServletRequest.class, HttpServletResponse.class);
            doPost.setAccessible(true);
            doPost.invoke(request,response);
         } catch (SecurityException e) {
            throw new ServletException(e);
         } catch (NoSuchMethodException e) {
            throw new ServletException(e);
         } catch (IllegalArgumentException e) {
            throw new ServletException(e);
         } catch (InstantiationException e) {
            throw new ServletException(e);
         } catch (IllegalAccessException e) {
            throw new ServletException(e);
         } catch (InvocationTargetException e) {
            throw new ServletException(e);
         }
         
      }
   }


   private Class getApplicationClass(HttpServletRequest request) throws ServletException { 
      if (httpServlet != null)
         return httpServlet;
      
      String applicationClassName;
      
      if (className != null)
         applicationClassName = className;
      else
         applicationClassName = request.getParameter("applicationClassName");
      
      
      
      if (applicationClassName == null) {
         return null;
//         throw new ServletException("ERROR: ServletForwarder expects the parameter applicationClassName to be specified and it wasn't.");
      }
      
      Class appClass;
      try {
         appClass = (Class)ReflectionCache.ClassCache.forName(Options.getCompleteClassName(applicationClassName));
      } catch (ClassNotFoundException e1) {
         throw new ServletException(e1);
      }
      return appClass;

   }
   

   public boolean outputHiddenIFrame(HttpServletRequest request, HttpServletResponse response) throws IOException {
      String iframePos = request.getParameter(Parameters.IFRAME_POS);
      String title = request.getParameter(Parameters.TITLE);
      if (iframePos != null) {
         // output hidden iframe contents
         response.setHeader("Cache-Control", "private");
         response.setHeader("Pragma", "no-cache");
         Writer out = response.getWriter();
         out.write("<html>\n");
         out.write("<head>");
         out.write("<script type=\"text/javascript\">parent.w20.updatingIframe = undefined;</script>\n");

         if (title != null) {
            out.write("<title>" + title + "</title>");
         }
         out.write("</head>\n");
         out.write("<body>");
         out.write("<input type=\"hidden\" id=\"" + Parameters.IFRAME_POS + "\" name=\"" + Parameters.IFRAME_POS + "\" value=\"" + iframePos + "\" />");
         out.write("</body>\n");
         out.write("</html>");
         
         return true;
      }
      return false;
   }
}
  