package com.getsnappy.snappyj;

public class StringUtil   {
   public static final String JAVA_SCRIPT_PREFIX = "J8v8sCr1pT:";
   /**
    * When passing Strings into js<...> methods, they will be passed through
    * jsQuote to be properly quoted.  Wrap the String with this function call,
    * and jsQuote will return the String as is.  This is useful when passing 
    * JavaScript to one of the js<...> methods.<br>
    * <br>
    * <b>For Example (this example will set and persist the value the input 
    *                 element (haha) to the calendarBean attribute TYPE):</b><br>
    * <code>out.write("<input type=\"radio\" value=\"<b>haha</b>\" onclick=\"" + calendarBean.jsSet(CategoryChooserBean.TYPE, StringUtil.dontQuote("this.value")) + "\">\n");</code>
    * 
    * @param in
    * @return
    */
   public static String dontQuote(String in) {
      return JAVA_SCRIPT_PREFIX + in;
      
   }
   
   /**
    * Take the given string and return a string in a format suitable for JavaScript.  Heres
    * the algoritm.  If the string was passed through of StringUtiluote, then it is returned 
    * as is.  If the string begin and ends with the single quote char, then is is returned
    * as is.  Otherwise this function adds a leading and trailing single qoute and escapes
    * both single quotes and double quotes with a forward slash.<br>
    * <br>
    * If you literally want the result to be a single quoted string ("'fdsdfsdfd'" or '\'fsdfsdfd\''),
    * you must pass explicitly pass '\'fsdfsdfd\''.
    * 
    * @param queryString
    * @return a string suitable for use in javascript.
    */
   public static String jsQuote(String string) {
      if (string == null) {
         return "null";         
      }
      if (string.length() == 0) {
         return "''";
      }
      if (string.startsWith(JAVA_SCRIPT_PREFIX)) {
         // FIXME: Yes I realize this isn't perfect.  Namely if someone happened
         // to pass a string in that began with JAVA_SCRIPT_PREFIX, it would
         // get treated as though dontQuote had been called and the prefix
         // would be stripped wrongly.
         string = string.substring(JAVA_SCRIPT_PREFIX.length());
         StringBuffer sb = new StringBuffer(string.length() + 16);
         for(int i = 0; i < string.length();i++) {
            char c = string.charAt(i);
            switch (c) 
            {
               case '"':
                  sb.append('\'');
                  break;
               default:
                  sb.append(c);
                  break;
            }
         }
         
         return sb.toString();

      }
      
      if (string.charAt(0) == '\'' && string.charAt(string.length() - 1) == '\'') {
         return string;
      }
      
      StringBuffer sb = new StringBuffer(string.length() + 16);
      sb.append('\'');
      for(int i = 0; i < string.length();i++) {
         char c = string.charAt(i);
         switch (c) 
         {
            case '\'':
            case '\"':
               sb.append('\\');
            default:
               sb.append(c);
               break;
         }
      }
      sb.append('\'');
      
      return sb.toString();
      
   }
   
   /**
    * Currently to calls the String version
    * 
    * @param string
    * @return
    */
   public static String jsQuote(Long l) {
      return jsQuote((l == null ? null : l.toString()));
   }
   
   /**
    * Currently to calls the String version
    * 
    * @param string
    * @return
    */
   public static String jsQuote(Integer i) {
      return jsQuote((i == null ? null : i.toString()));
   }
   
   /**
    * Returns either undefined, true or false
    * 
    * @param string
    * @return
    */
   public static String jsQuote(Boolean b) {
      return (b == null ? "undefined" : b.toString());
   }

}
